@@ -12,3 +12,11 @@ class RemoteExecutionError(RemoteError):
 
 class ConfigurationError(RemoteError):
     """The workspace configuration is incorrect"""
+
+
+class InvalidInputError(RemoteError):
+    """Invalid user input is passed from the cli"""
+
+
+class InvalidRemoteHostLabel(RemoteError):
+    """Invalid label is passed to the workspace."""
