@@ -106,7 +106,9 @@ def test_remote_init(mock_run, tmp_path):
     assert "Created remote directory at test-host.example.com:.remotes/myproject_" in result.output
     assert "Remote is configured and ready to use" in result.output
 
-    mock_run.assert_called_once_with(["ssh", "-tKq", "test-host.example.com", ANY], stdin=ANY, stdout=ANY, stderr=ANY)
+    mock_run.assert_called_once_with(
+        ["ssh", "-tKq", "-o", "BatchMode=yes", "test-host.example.com", ANY], stdin=ANY, stdout=ANY, stderr=ANY
+    )
 
     assert (subdir / CONFIG_FILE_NAME).exists()
     assert (subdir / CONFIG_FILE_NAME).read_text().startswith("test-host.example.com:.remotes/myproject_")
@@ -144,7 +146,7 @@ Remote is configured and ready to use
     )
 
     mock_run.assert_called_once_with(
-        ["ssh", "-tKq", "test-host.example.com", "mkdir -p .path/test.dir/_test-dir"],
+        ["ssh", "-tKq", "-o", "BatchMode=yes", "test-host.example.com", "mkdir -p .path/test.dir/_test-dir"],
         stdin=ANY,
         stdout=ANY,
         stderr=ANY,
@@ -259,7 +261,7 @@ def test_remote_add_adds_host(mock_run, tmp_workspace):
     assert (tmp_workspace / CONFIG_FILE_NAME).read_text() == f"{TEST_CONFIG}\nhost:directory\n"
 
     mock_run.assert_called_once_with(
-        ["ssh", "-tKq", "host", "mkdir -p directory"], stdin=ANY, stdout=ANY, stderr=ANY,
+        ["ssh", "-tKq", "-o", "BatchMode=yes", "host", "mkdir -p directory"], stdin=ANY, stdout=ANY, stderr=ANY,
     )
 
 
@@ -345,7 +347,7 @@ def test_remote_set(tmp_workspace):
         bad_attempt_result = runner.invoke(entrypoints.remote_set, ["10"])
 
     assert set_result.exit_code == 0
-    assert host_result.output == f"new-host\n"
+    assert host_result.output == "new-host\n"
     assert bad_attempt_result.exit_code == 1
     assert (tmp_workspace / INDEX_FILE_NAME).read_text() == "2\n"
 
@@ -365,11 +367,14 @@ def test_remote(mock_run, tmp_workspace):
             call(
                 [
                     "rsync",
-                    "-rlpmchz",
+                    "-arlpmchz",
                     "--copy-unsafe-links",
                     "-e",
-                    "ssh -q",
+                    "ssh -Kq -o BatchMode=yes",
                     "--force",
+                    "--delete",
+                    "--rsync-path",
+                    "mkdir -p .remotes/myproject && rsync",
                     "--exclude-from",
                     ANY,
                     f"{tmp_workspace}/",
@@ -382,6 +387,8 @@ def test_remote(mock_run, tmp_workspace):
                 [
                     "ssh",
                     "-tKq",
+                    "-o",
+                    "BatchMode=yes",
                     TEST_HOST,
                     """if [ -f .remotes/myproject/.remoteenv ]; then
   source .remotes/myproject/.remoteenv 2>/dev/null 1>/dev/null
@@ -397,10 +404,10 @@ echo test >> .file
             call(
                 [
                     "rsync",
-                    "-rlpmchz",
+                    "-arlpmchz",
                     "--copy-unsafe-links",
                     "-e",
-                    "ssh -q",
+                    "ssh -Kq -o BatchMode=yes",
                     "--force",
                     "--exclude-from",
                     ANY,
@@ -429,11 +436,14 @@ def test_remote_execution_fail(mock_run, tmp_workspace):
             call(
                 [
                     "rsync",
-                    "-rlpmchz",
+                    "-arlpmchz",
                     "--copy-unsafe-links",
                     "-e",
-                    "ssh -q",
+                    "ssh -Kq -o BatchMode=yes",
                     "--force",
+                    "--delete",
+                    "--rsync-path",
+                    "mkdir -p .remotes/myproject && rsync",
                     "--exclude-from",
                     ANY,
                     f"{tmp_workspace}/",
@@ -446,6 +456,8 @@ def test_remote_execution_fail(mock_run, tmp_workspace):
                 [
                     "ssh",
                     "-tKq",
+                    "-o",
+                    "BatchMode=yes",
                     TEST_HOST,
                     """if [ -f .remotes/myproject/.remoteenv ]; then
   source .remotes/myproject/.remoteenv 2>/dev/null 1>/dev/null
@@ -461,10 +473,10 @@ echo 'test >> .file'
             call(
                 [
                     "rsync",
-                    "-rlpmchz",
+                    "-arlpmchz",
                     "--copy-unsafe-links",
                     "-e",
-                    "ssh -q",
+                    "ssh -Kq -o BatchMode=yes",
                     "--force",
                     "--exclude-from",
                     ANY,
@@ -491,11 +503,14 @@ def test_remote_sync_fail(mock_run, tmp_workspace):
     mock_run.assert_called_once_with(
         [
             "rsync",
-            "-rlpmchz",
+            "-arlpmchz",
             "--copy-unsafe-links",
             "-e",
-            "ssh -q",
+            "ssh -Kq -o BatchMode=yes",
             "--force",
+            "--delete",
+            "--rsync-path",
+            "mkdir -p .remotes/myproject && rsync",
             "--exclude-from",
             ANY,
             f"{tmp_workspace}/",
@@ -519,6 +534,8 @@ def test_remote_quick(mock_run, tmp_workspace):
         [
             "ssh",
             "-tKq",
+            "-o",
+            "BatchMode=yes",
             TEST_HOST,
             """if [ -f .remotes/myproject/.remoteenv ]; then
   source .remotes/myproject/.remoteenv 2>/dev/null 1>/dev/null
@@ -546,6 +563,8 @@ def test_remote_quick_execution_fail(mock_run, tmp_workspace):
         [
             "ssh",
             "-tKq",
+            "-o",
+            "BatchMode=yes",
             TEST_HOST,
             """if [ -f .remotes/myproject/.remoteenv ]; then
   source .remotes/myproject/.remoteenv 2>/dev/null 1>/dev/null
@@ -572,12 +591,15 @@ def test_remote_push(mock_run, tmp_workspace):
     mock_run.assert_called_once_with(
         [
             "rsync",
-            "-rlpmchz",
+            "-arlpmchz",
             "--copy-unsafe-links",
             "-e",
-            "ssh -q",
+            "ssh -Kq -o BatchMode=yes",
             "--force",
             "-i",
+            "--delete",
+            "--rsync-path",
+            "mkdir -p .remotes/myproject && rsync",
             "--exclude-from",
             ANY,
             f"{tmp_workspace}/",
@@ -605,12 +627,15 @@ def test_remote_push_mass(mock_run, tmp_workspace):
             call(
                 [
                     "rsync",
-                    "-rlpmchz",
+                    "-arlpmchz",
                     "--copy-unsafe-links",
                     "-e",
-                    "ssh -q",
+                    "ssh -Kq -o BatchMode=yes",
                     "--force",
                     "-i",
+                    "--delete",
+                    "--rsync-path",
+                    "mkdir -p .remotes/myproject && rsync",
                     "--exclude-from",
                     ANY,
                     f"{tmp_workspace}/",
@@ -622,16 +647,19 @@ def test_remote_push_mass(mock_run, tmp_workspace):
             call(
                 [
                     "rsync",
-                    "-rlpmchz",
+                    "-arlpmchz",
                     "--copy-unsafe-links",
                     "-e",
-                    "ssh -q",
+                    "ssh -Kq -o BatchMode=yes",
                     "--force",
                     "-i",
+                    "--delete",
+                    "--rsync-path",
+                    "mkdir -p other-directory && rsync",
                     "--exclude-from",
                     ANY,
                     f"{tmp_workspace}/",
-                    f"new-host:other-directory",
+                    "new-host:other-directory",
                 ],
                 stdout=ANY,
                 stderr=ANY,
@@ -652,10 +680,10 @@ def test_remote_pull(mock_run, tmp_workspace):
     mock_run.assert_called_once_with(
         [
             "rsync",
-            "-rlpmchz",
+            "-arlpmchz",
             "--copy-unsafe-links",
             "-e",
-            "ssh -q",
+            "ssh -Kq -o BatchMode=yes",
             "--force",
             "-i",
             "--exclude-from",
@@ -683,14 +711,14 @@ def test_remote_pull_subdirs(mock_run, tmp_workspace):
             call(
                 [
                     "rsync",
-                    "-rlpmchz",
+                    "-arlpmchz",
                     "--copy-unsafe-links",
                     "-e",
-                    "ssh -q",
+                    "ssh -Kq -o BatchMode=yes",
                     "--force",
                     "-i",
                     f"{TEST_HOST}:{TEST_DIR}/build",
-                    f"{tmp_workspace}/build",
+                    f"{tmp_workspace}/",
                 ],
                 stdout=ANY,
                 stderr=ANY,
@@ -698,14 +726,14 @@ def test_remote_pull_subdirs(mock_run, tmp_workspace):
             call(
                 [
                     "rsync",
-                    "-rlpmchz",
+                    "-arlpmchz",
                     "--copy-unsafe-links",
                     "-e",
-                    "ssh -q",
+                    "ssh -Kq -o BatchMode=yes",
                     "--force",
                     "-i",
                     f"{TEST_HOST}:{TEST_DIR}/dist",
-                    f"{tmp_workspace}/dist",
+                    f"{tmp_workspace}/",
                 ],
                 stdout=ANY,
                 stderr=ANY,
@@ -724,5 +752,61 @@ def test_remote_delete(mock_run, tmp_workspace):
 
     assert result.exit_code == 0
     mock_run.assert_called_once_with(
-        ["ssh", "-tKq", TEST_HOST, f"rm -rf {TEST_DIR}"], stdin=ANY, stdout=ANY, stderr=ANY,
+        ["ssh", "-tKq", "-o", "BatchMode=yes", TEST_HOST, f"rm -rf {TEST_DIR}"], stdin=ANY, stdout=ANY, stderr=ANY,
     )
+
+
+@pytest.mark.parametrize(
+    "port_value, expected_output, expected_exit_code",
+    [
+        ("bar:foo", "Please pass valid integer value for ports", 1),
+        ("bar:5000", "Please pass valid integer value for ports", 1),
+        ("bar:foo:foo", "Please pass a valid value to enable local port forwarding", 1),
+        ("2.4:2.4", "Please pass valid integer value for ports", 1),
+    ],
+)
+@patch("remote.util.subprocess.run")
+def test_remote_port_forwarding_user_input_error(
+    mock_run, tmp_workspace, port_value, expected_output, expected_exit_code
+):
+    mock_run.return_value = Mock(returncode=0)
+    runner = CliRunner()
+    with cwd(tmp_workspace):
+        result = runner.invoke(entrypoints.remote, ["-t", port_value, "echo test"])
+        assert result.exit_code == expected_exit_code
+        assert expected_output in result.output
+
+
+@pytest.mark.parametrize(
+    "port_value, expected_port_forwarding, expected_exit_code",
+    [("5000", "5000:localhost:5000", 0), ("5000:5005", "5005:localhost:5000", 0)],
+)
+@patch("remote.util.subprocess.run")
+def test_remote_port_forwarding_successful(
+    mock_run, tmp_workspace, port_value, expected_port_forwarding, expected_exit_code
+):
+    mock_run.return_value = Mock(returncode=0)
+    runner = CliRunner()
+    with cwd(tmp_workspace):
+        result = runner.invoke(entrypoints.remote, ["-t", port_value, "echo test"])
+        assert result.exit_code == expected_exit_code
+        mock_run.assert_any_call(
+            [
+                "ssh",
+                "-tKq",
+                "-o",
+                "BatchMode=yes",
+                "-L",
+                expected_port_forwarding,
+                "test-host1.example.com",
+                """if [ -f .remotes/myproject/.remoteenv ]; then
+  source .remotes/myproject/.remoteenv 2>/dev/null 1>/dev/null
+fi
+cd .remotes/myproject
+echo test
+""",
+            ],
+            stderr=ANY,
+            stdin=ANY,
+            stdout=ANY,
+        )
